/* SF_Multiplier class declaration
   Copyright (C) 1998--1999 Jarno Seppnen and Sami Kananoja
   $Id: multiplier.hh,v 1.3 1999/03/22 11:19:45 jams Exp $

   This file is part of Sonic Flow.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

#ifndef __SF_MULTIPLIER_HH__
#define __SF_MULTIPLIER_HH__

#include <sf/block.hh>
#include <sf/input_terminal.hh>
#include <sf/output_terminal.hh>

class SF_Multiplier : public SF_Block
{
public:
    SF_Multiplier (const char* name = 0);
    virtual ~SF_Multiplier ();
    virtual void finish ();
    virtual void initialize ();
    virtual void execute ();

    inline void set_initial_value (SF_Sample a);
    inline SF_Sample get_initial_value ();

protected:
    // Constant value which the product is multiplied with; 1 by default
    SF_Sample initial_value;
    // Pointer to the signal in the frame of the output terminal "y"
    SF_Sample* s_y;
    // Pointers to the input and output terminals; the terminals are addressed
    // through these pointers in the execute () call instead of fetching the
    // pointers by name
    SF_Input_Terminal* in_x;
    SF_Output_Terminal* out_y;

private:
    //Disable copy constructor and operator=
    SF_Multiplier (const SF_Multiplier& a);
    SF_Multiplier& operator= (const SF_Multiplier& a);
};

inline void
SF_Multiplier::set_initial_value (SF_Sample a)
{
    initial_value = a;
}

inline SF_Sample
SF_Multiplier::get_initial_value ()
{
    return initial_value;
}

#endif

/* EOF */
