/* SF_Band_Pass_Filter
   Copyright (C) 1998--1999 Jarno Seppnen and Sami Kananoja
   $Id: band_pass_filter.cc,v 1.3 1999/03/22 11:20:09 jams Exp $

   This file is part of Sonic Flow.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA. */

#include <sf/block.hh>
#include <sf/exception.hh>
#include <sf/frame.hh>
#include <sf/input_terminal.hh>
#include <sf/output_terminal.hh>
#include <sf/typedefs.h>

#include <sf/blocks/band_pass_filter.hh>

SF_Band_Pass_Filter::SF_Band_Pass_Filter (const char* name)
    : SF_Biquad_Filter (false,
			false,
			"Band-pass filter",
			name),
      s_f1 (0), s_f2 (0)
{
    in_f1 = new SF_Input_Terminal ("f1", this, false);
    in_f2 = new SF_Input_Terminal ("f2", this, false);
    add_input_terminal (in_f1);
    add_input_terminal (in_f2);
}

SF_Band_Pass_Filter::~SF_Band_Pass_Filter ()
{
    remove_input_terminal (in_f1);
    remove_input_terminal (in_f2);
    delete in_f1;
    delete in_f2;
    in_f1 = 0;
    in_f2 = 0;
}

void
SF_Band_Pass_Filter::initialize ()
{
    SF_Biquad_Filter::initialize ();
    s_f1 = in_f1->get_source_frame (0)->get_signal ();
    s_f2 = in_f2->get_source_frame (0)->get_signal ();
}

void
SF_Band_Pass_Filter::execute ()
{
    SF_Biquad_Filter::execute ();
    SF_Length l = out_y->get_frame ()->get_num_rows ();
    for (int i = 0; i < l; i++)
    {
	if (s_f1[i] >= s_f2[i])
	{
	    throw SF_Exception (this, "Lower frequency must be less than the upper frequency");
	}
	if (s_f1[i] < 0)
	{
	    throw SF_Exception (this, "Frequencies must be non-negative");
	}	   
	if (s_f2[i] > (fs / 2))
	{
	    throw SF_Exception (this, "Frequencies must be less than or equal to half of the sample frequency");
	}	   
	calculate_coefficients (fs, s_f1[i], s_f2[i]);
	s_y[i] = filter (s_x[i]);
    }
}

void
SF_Band_Pass_Filter::finish ()
{
    SF_Biquad_Filter::finish ();
}

/* EOF */
